import sys
import os
from pathlib import Path
from .utils import *
import jupytext
from nbconvert.preprocessors import ExecutePreprocessor
from nbconvert.exporters import HTMLExporter
from traitlets.config import Config


# Configure and run out exporter


def generate(source = None, dest = None, report = None):
  """
  generate jypyter HTML reports from a folder with markdown files
  python blocs can be evaluated with the the follwing syntax
  ```python
  print("hello")
  ```
  source: de folder containing the st files to transform into html files
  dest: the destination folder to store destination HTML files 
  """
  #ensuring source & destination folder exists 
  pkg_dir, this_filename = os.path.split(__file__)
  source = os.path.join(pkg_dir, "data/reports") if source == None else source 
  dest = os.path.join(get_home(), "reports") if dest == None else dest
  Path(source).mkdir(parents=False, exist_ok=True)
  Path(dest).mkdir(parents=False, exist_ok=True)

  #setting up the jupyter kernel name
  ep = ExecutePreprocessor(timeout=600, kernel_name='python3')
  c = Config()

  # Configure our tag removal
  c.TagRemovePreprocessor.remove_cell_tags = ("hide-cell",)
  c.TagRemovePreprocessor.remove_all_outputs_tags = ('hide-output',)
  c.TagRemovePreprocessor.remove_input_tags = ('hide-input',)
  c.HTMLExporter.preprocessors = ["nbconvert.preprocessors.TagRemovePreprocessor"]

  html_exporter = HTMLExporter(config=c)
  #html_exporter.template_name = 'classic'
  
  #iterating over all reports
  for path in os.listdir(source):
    if path.endswith('.md'):
      report_name = path[0:len(path)-3]
      if report == None or report_name == report:
        print(f"generating report {report_name}")
        nb = jupytext.read(os.path.join(source, path))   
        ep.preprocess(nb)             
        (body, resources) = html_exporter.from_notebook_node(nb)
        with open(os.path.join(dest, f"{report_name}.html"), 'w') as out:
          out.write(body)
  
  exec_env = {} 




import sys
import argparse
from tkinter import Tk
from tkinter.filedialog import askopenfilename
from .p02_initial_extraction import *
from . import utils

def main(a):
  # Base argument parser
  parser = argparse.ArgumentParser()
  subs = parser.add_subparsers()
  
  # extract command
  extract_parser = subs.add_parser("extract", help = "Invoke initial extractions commands") 
  extract_subs = extract_parser.add_subparsers()

  # extract neoscope command
  neoextract_parser = extract_subs.add_parser("neoscope", help = "Invoke neoscope extractions commands") 
  neoextract_subs = neoextract_parser.add_subparsers()

  # -- get neoscope key to clipboard
  neokey2clipboard_parser = neoextract_subs.add_parser("readkey", aliases=['key'], help = "Copy the neoscope extraction key from qrcode to clipboard")
  neokey2clipboard_parser.set_defaults(func = do_neokey2clipboard)

  # -- encrypt neoscope extractions
  encrypt_neoextract_parser = neoextract_subs.add_parser("encrypt", help = "Encrypt neoscope extractions")
  encrypt_neoextract_parser.add_argument("-e", "--extractions", required=False, help="Path to a file containing the extraction files to be encryped. If not provided it will be prompted")
  encrypt_neoextract_parser.add_argument("-w", "--webcam", required=False, help="Flag to establish that system webcam will be used to get password", action = "store_true", default = False)
  encrypt_neoextract_parser.add_argument("-c", "--clipboard", required=False, help="Flag to establish that system clipboard will be used to get password", action = "store_true", default = True)
  encrypt_neoextract_parser.set_defaults(func = do_encrypt_neoscope_extractions)

  # -- upload encrypted files to epifiles
  neoextract2epifiles_parser = neoextract_subs.add_parser(
    "push2epifiles", 
    aliases=["push"], 
    help = "Push neoscope extractions to epifiles. Password will be requested or can be set on environmant variable DP_PWD_EPI_FILES"
  )
  neoextract2epifiles_parser.add_argument(
    "-e", 
    "--epifiles-host", 
    required=False, 
    help="epifiles base host, default = epifiles.voozanoo.net", 
    default = "epifiles.voozanoo.net"
  )
  neoextract2epifiles_parser.add_argument(
    "-r", 
    "--maven-repo", 
    required=False, 
    help="maven repository to use to download sparkly and dependencies, default = https://repo1.maven.org/maven2", 
    default = "https://repo1.maven.org/maven2"
  )
  neoextract2epifiles_parser.add_argument("-u", "--epi-user", required=True, help="login for connecting to epifiles")
  neoextract2epifiles_parser.set_defaults(func = do_neoextract2epifiles)

  # -- download encrypted files from epifiles
  epifiles2neoextract_parser = neoextract_subs.add_parser("pull2bigdata", aliases=["pull"], help = "Pull neoscope extractions from epifiles to big data infrastructure")
  epifiles2neoextract_parser.add_argument(
    "-e", 
    "--epifiles-host", 
    required=False, 
    help="epifiles base host, default = epifiles.voozanoo.net", 
    default = "epifiles.voozanoo.net"
  )
  epifiles2neoextract_parser.add_argument(
    "-r", 
    "--maven-repo", 
    required=False, 
    help="maven repository to use to download sparkly and dependencies, default = https://repo1.maven.org/maven2", 
    default = "https://repo1.maven.org/maven2"
  )
  epifiles2neoextract_parser.add_argument("-u", "--epi-user", required=True, help="login for connecting to epifiles")
  epifiles2neoextract_parser.set_defaults(func = do_epifiles2bigdata)
  
  # -- decrypt neoscope extractions
  decrypt_neoextract_parser = neoextract_subs.add_parser("decrypt", help = "Decrypt neoscope extractions")
  decrypt_neoextract_parser.set_defaults(func = do_decrypt_neoscope_extractions)
   
  # extract esis command
  esis_parser = extract_subs.add_parser("esis", help = "Invoke esis command") 
  esis_subs = esis_parser.add_subparsers()

  # -- get esis data dicom ids
  esis_get_guid = esis_subs.add_parser("dicom-guid", aliases=['guid'], help = "Get the list of dicom guids. Password will be requested or can be set on environmant variable DP_PWD_ESIS")
  esis_get_guid.add_argument(
    "-e", 
    "--esis-host", 
    required=False, 
    help="esis application host, default = https://neoesis.preprod.voozanoo.net/neodemat", 
    default = "https://neoesis.preprod.voozanoo.net/neodemat"
  )
  esis_get_guid.add_argument(
    "-n", 
    "--data-query", 
    required=False, 
    help="name or id of the data query to extract the guids", 
    default = "deep-piste-extract"
  )
  esis_get_guid.add_argument("-u", "--esis-user", required=True, help="login for connecting to esis")
  esis_get_guid.add_argument("-b", "--batch-size", required=False, help="batch size for data query pooling")
  esis_get_guid.add_argument("-r", "--remote-dest", required=False, help="remote destination if file has to sent to a remote server via ssh")
  esis_get_guid.set_defaults(func = do_get_dicom_guid)
  
  # -- analyse esis dicom ids
  esis_reports = esis_subs.add_parser("report", help = "Produce a set of aggregation to manually validate de soundness of esis DICOM id validations")
  esis_reports.set_defaults(func = do_esis_report)

  # extract dcm4chee command
  dcm4chee_parser = extract_subs.add_parser("dcm4chee", help = "Invoke dcm4chee extractions commands") 
  dcm4chee_subs = dcm4chee_parser.add_subparsers()

  # -- get dicom parser
  get_dicom_parser = dcm4chee_subs.add_parser("dicom", help = "Get dicom files from dcm4chee")
  get_dicom_parser.add_argument("-s", "--server", required=True, help="Path to a file containing the extraction files to be encryped. If not provided it will be prompted")
  get_dicom_parser.add_argument("-p", "--port", required=False, help="Flag to establish that system webcam will be used to get password", default = 11112)
  get_dicom_parser.set_defaults(func = do_get_dicom)
  
  #calling handlers
  func = None
  try:
    args = parser.parse_args()
    func = args.func
  except AttributeError:
    parser.print_help()
  if func != None:
    args.func(args, parser)
  

# handlers
def do_neokey2clipboard(args, *other):
  p02_002_neoscope_key_to_clipboard()

def do_encrypt_neoscope_extractions(args, *other):
  get_home()
  if args.extractions == None:
    print("Please chose the file to crypt")
    Tk().withdraw()
    filename = askopenfilename()
  else:
    filename = args.extractions
  p02_003_encrypt_neoscope_extractions(source = filename, webcam_pwd = args.webcam, clipboard_pwd = args.clipbord)   

def do_neoextract2epifiles(args, *other):
  utils.prepare_sparkly(repo = args.maven_repo)
  p02_004_send_neoscope_extractions_to_epifiles(
    epifiles = args.epifiles_host, 
    login=args.epi_user, 
    password=utils.get_password(f"epifiles", f"Password for {args.epi_user}")
  )
def do_epifiles2bigdata(args, *other):
  utils.prepare_sparkly(repo = args.maven_repo)
  p02_005_get_neoscope_extractions_from_epifiles(
    epifiles = args.epifiles_host, 
    login=args.epi_user, 
    password=utils.get_password(f"epifiles", f"Password for {args.epi_user}")
  )
def do_decrypt_neoscope_extractions(args, *other):
  p02_006_decrypt_neoscope_extractions(key = utils.get_password(f"neo_decrypt", f"Key for Neoscope extractions"))   

def do_get_dicom_guid(args, *other):
  p02_007_get_dicom_guid(
    esis_host = args.esis_host, 
    dataquery = args.data_query, 
    login=args.esis_user, 
    password = utils.get_password(f"esis", f"Esis password for getting dicom guid queries"), 
    batch_size = args.batch_size,
    remote_dest = args.remote_dest
  )   
def do_get_dicom(args, *other):
  p02_008_get_dicom(server = args.server, port = args.port)

def do_esis_report(args, *other):
  p02_010_dicom_guid_report() 

if __name__ == "__main__":
  main(sys.argv[1] if len(sys.argv)>1 else None)





import kskit
import os
import base64
import clipboard
import urllib.parse
from tkinter import Tk
from tkinter import filedialog
from .utils import get_home, sparkly_cp
from kskit import voo
import kskit.crypto
import kskit.dicom
import tempfile
import subprocess
import pandas as pd
import numpy as np
from dpiste import report

def p02_001_generate_neoscope_key():
  """Generate a 256bit random QR code to be used as an AES encryption simmetric key"""
  dp_home = get_home()
  neo_key_path = os.path.join(dp_home, "neo_key.png")
  kskit.generate_qr_key(neo_key_path, 32) #32 bytes = 256bits
  print(f"Neoscope extractios key produced on '{neo_key_path}'. Please print too copies of this file, send one to Neoscope operator end delete this file")
  
def p02_002_neoscope_key_to_clipboard():   
  """Use user webcam to copy the neoscope encryption base 64 key to the clipboard"""
  b64key = kskit.crypto.read_webcam_key(auto_close = True, camera_index = 0)
  clipboard.copy(b64key)
  print(f"the key has been copied to the clipboard, put it to the server clipboard and call p02_003_encrypt_neoscope_extractions on it")

def p02_003_encrypt_neoscope_extractions(source, webcam_pwd = True, clipboard_pwd = False):   
  """Encrypts the provided file to the dp_home directory using a clipboard pasted base 64 key"""
  dest = os.path.join(get_home(), "extraction_neoscope.aes")
  if webcam_pwd:
    b64key = kskit.crypto.read_webcam_key(auto_close = True, camera_index = 0)
  elif clipboard_pwd:
    b64key = clipboard.paste()
  else:
    raise ValueError("either webcam or clipboard password has to be usef")
  kskit.encrypt(source, dest, b64key)
  #cleaning clipboard
  clipboard.copy("")
  print(f"file has been encrypted and stored on {dest} please proceed delete {source}, send to epiconcept via Epifiles and then remove {dest} too")

def p02_004_send_neoscope_extractions_to_epifiles(epifiles, login, password):
  """Sends the encrypted neoscope extractions to epifiles"""
  source = os.path.join(get_home(), "extraction_neoscope.aes")
  dest = f"epi://{urllib.parse.quote_plus(login)}:{urllib.parse.quote_plus(password)}@{epifiles}/extraction_neoscope.aes"
  sparkly_cp(source = source, dest = dest) 

def p02_005_get_neoscope_extractions_from_epifiles(epifiles, login, password):
  """Gets the encrypted neoscope extractions from epifiles"""
  dest = os.path.join(get_home(), "extraction_neoscope.aes")
  source = f"epi://{urllib.parse.quote_plus(login)}:{urllib.parse.quote_plus(password)}@{epifiles}/extraction_neoscope.aes"
  sparkly_cp(source = source, dest = dest) 

def p02_006_decrypt_neoscope_extractions(key):   
  """Decrypts the provided file to the dp_home directory using a key pasted on the clipboard"""
  crypted = os.path.join(get_home(), "extraction_neoscope.aes")
  orig = os.path.join(get_home(), "extraction_neoscope.zip")
  b64key = key 
  kskit.decrypt(crypted, orig, b64key)
  print(f"file has been decrypted and stored on {orig} please proceed delete {crypted} and run the extractions")

def p02_007_get_dicom_guid(esis_host, dataquery, login, password, batch_size, remote_dest = None):
  """
    Executes the esis data query for getting dicomq guids
    data query source code is located here: https://github.com/Epiconcept-Paris/deep.piste/blob/main/dpiste/data/esis.xml
  """
  tdir = tempfile.TemporaryDirectory()
  ldest = os.path.join(get_home() if remote_dest == None else tdir.name, "esis_dicom_guid.parquet") 
  try:
    df = voo.get_dataset(
      voo_url=esis_host, 
      login=login, 
      password=password, 
      dataset = dataquery, 
      format = "json", 
      order_by = None, 
      batch = batch_size)
    df.to_parquet(ldest, "pyarrow")
    if remote_dest != None:
      subprocess.run(["scp", ldest, f"{remote_dest}:{os.path.join(get_home(), 'esis_dicom_guid.parquet')}"])
  finally:
    tdir.cleanup()

def p02_008_get_dicom(server, port = 11112, retrieveLevel = 'SERIES'):
  ids = '1.3.6.1.4.1.9590.100.1.2.25530538411839768118282182472795553760'
  title = "DCM4CHEE"
  dest = os.path.join(get_home(), "dicom", ids) 
  if not os.path.exists(dest):
    os.makedirs(dest)
   
  kskit.dicom.get_dicom(ids = ids, dest = dest, server = server, port = port, title = title, retrieveLevel = 'SERIES')

def p02_009_neo_stats():
  raise NotImplementedError

def p02_010_dicom_guid_report():
  report.generate(report = "esis-import")

def p02_011_dicom_stats():
  raise NotImplementedError
